package api

import (
	"encoding/json"
	"errors"
	"fmt"
	"os"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func CreateMaterial(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	material := &model.ImageMaterial{}
	err := c.ShouldBindJSON(&material)
	if err != nil {
		return nil, err
	}

	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionImageMaterial, &material)
}

func DeleteMaterial(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionImageMaterial, _id)
}

func MaterialList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	if v, ok := query["name"]; ok {
		name := v.(string)
		if len(name) > 0 {
			delete(query, "name")
			query["attributes.name"] = bson.M{"$regex": name, "$options": "$i"}
		}
	}

	result, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionImageMaterial,
		Page:        page,
		Size:        size,
		Query:       query,
	})

	return result, err
}

func UpdateMaterial(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	material := &model.ImageMaterial{}
	err := c.ShouldBindJSON(&material)
	if err != nil {
		return nil, err
	}
	if material.Id.IsZero() {
		return nil, errors.New("id错误")
	}

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionImageMaterial, material.Id.Hex(), material)
}

func MaterialInsert(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// 读取json文件
	jsonData, err := os.ReadFile("api/datalist/11卡证背景.json")
	if err != nil {
		return nil, fmt.Errorf("failed to read JSON file: %v", err)
	}

	var jsonMaterials []*model.ImageMaterial
	if err := json.Unmarshal(jsonData, &jsonMaterials); err != nil {
		return nil, fmt.Errorf("failed to parse JSON: %v", err)
	}
	fmt.Println("total:", len(jsonMaterials))
	count := 0
	// 转换为model.ImageMaterial结构并插入数据库
	for _, m := range jsonMaterials {
		m.Id = primitive.NewObjectID()
		m.Type = 11
		id, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionImageMaterial, m)
		if err != nil {
			fmt.Println(err)
		} else {
			fmt.Println(id)
			count++

		}
	}

	return count, nil
}
