package api

import (
	"sku3dweb/bus"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func ServiceLoginSucc(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	//登录成功
	setting := &model.UserSetting{}

	ret, err := repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), repo.CollectionUserSetting, apictx.User.ID, bson.M{"$set": bson.M{"lastLoginTime": time.Now()}})
	if err != nil {
		return nil, err
	}

	if ret.MatchedCount == 0 {
		setting.Id, _ = primitive.ObjectIDFromHex(apictx.User.ID)
		setting.LoginName = apictx.User.Phone
		setting.CreateTime = time.Now()
		setting.LastLoginTime = time.Now()

		info, err := bus.GetUserInfo(apictx.User.ID)
		if err != nil {
			return nil, err
		}
		setting.LoginName = info.Name
		setting.UserType = *info.UserType
		setting.Avator = info.Avatar

		return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionUserSetting, setting)
	}

	return true, nil
}

func ServiceUserProfile(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	user := apictx.User

	ok, u := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionAdmin,
		Project:     []string{"name", "phone", "isRoot", "perms", "lastLogin"},
		Query:       repo.Map{"_id": user.ID},
	})
	if !ok {
		return nil, NewError("用户不存在")
	}
	out := map[string]interface{}{
		"user": u,
	}
	return out, nil
}

func RegAdminRouter(router *GinRouter) {

	router.GET("/admin/user/list", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		page, size, query := UtilQueryPageSize(c)

		pageOption := &repo.PageSearchOptions{
			CollectName: repo.CollectionUserSetting,
			Page:        page,
			Size:        size,
			Query:       query,
			Project:     []string{"loginName", "userType", "avator", "createTime", "updateTime", "enableMall"},
		}

		return repo.RepoPageSearch(apictx.CreateRepoCtx(), pageOption)
	})

	// CreateCRUD(router, "/admin/user", &CRUDOption{
	// 	Collection: repo.CollectionAdmin,
	// 	NewModel2: func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// 		body := &struct {
	// 			Phone    string
	// 			Password string
	// 			Perms    string
	// 			IsRoot   bool
	// 			Name     string
	// 		}{}
	// 		c.ShouldBindJSON(&body)

	// 		ok, _ := repo.RepoSeachDocMap(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
	// 			CollectName: repo.CollectionAdmin,
	// 			Query:       repo.Map{"phone": body.Phone},
	// 			Project:     []string{"_id"},
	// 		})
	// 		if ok {
	// 			return nil, nil
	// 		}

	// 		return &model.Admin{
	// 			Name:       body.Name,
	// 			IsRoot:     &body.IsRoot,
	// 			Phone:      body.Phone,
	// 			Password:   UtilMd5(body.Password),
	// 			Perms:      body.Perms,
	// 			CreateTime: time.Now(),
	// 		}, nil
	// 	},
	// 	EmtyModel: func(*gin.Context) interface{} {
	// 		return &model.Admin{}
	// 	},
	// 	Update: func(c *gin.Context, apictx *ApiSession, entity interface{}, id string) {
	// 		m := entity.(*model.Admin)
	// 		if len(m.Password) > 0 {
	// 			m.Password = UtilMd5(m.Password)
	// 		}
	// 	},
	// 	SearchProject: []string{"phone", "name", "lastLogin", "perms", "isRoot"},
	// })
}
