package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Submit(c *gin.Context, apictx *ApiSession, fromType, fromId, fromScope string, ToScope string) (interface{}, error) {

	userCollections := map[string]string{"fabric": repo.CollectionFabric, "imgmat": repo.CollectionImageMat, "sticker": repo.CollectionStickers, "design": repo.CollectionDesigns, "last": repo.CollectionLasts, "heel": repo.CollectionHeels, "sole": repo.CollectionSoles, "decorate": repo.CollectionDecorates}
	srcCollection := userCollections[fromType]

	if fromScope == "team" {
		srcCollection = fmt.Sprintf("%s_%s", fromType, fromScope)
	}

	if len(srcCollection) < 1 {
		return nil, NewError("库类型不支持!")
	}

	targetCollection := fmt.Sprintf("%s_%s", fromType, ToScope)

	teamId := repo.TeamGetUserTeamId(apictx.CreateRepoCtx(), apictx.User.ID)
	if len(teamId) < 1 && ToScope == "team" {
		return nil, NewError("当前用户没有加入任何部门!")
	}

	if fromType == "design" {
		entity := &model.Design{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}
		entity.CreateTime = time.Now()
		entity.Sharepwd = ""
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = Int32Value(model.LibState_Complated) //发布默认是完成

		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "last" {
		entity := &model.LastMesh{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成
		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "heel" {
		entity := &model.HeelMesh{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成
		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "sole" {
		entity := &model.SoleMesh{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成

		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "decorate" {
		entity := &model.DecorateMesh{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成
		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "imgmat" {
		entity := &model.ImageMat{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成

		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "fabric" {
		entity := &model.Fabric{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成

		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	if fromType == "sticker" {
		entity := &model.StickerImage{}
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: srcCollection,
			Query:       repo.Map{"_id": fromId},
		}, entity)
		if err != nil || !ok {
			return nil, NewError("获取当前资源出错!")
		}

		entity.CreateTime = time.Now()
		entity.Id = primitive.NilObjectID
		entity.TeamId = teamId
		entity.CompanyId = apictx.User.Parent
		entity.State = model.LibState_Complated //发布默认是完成

		//插入目标库
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), targetCollection, entity)
	}

	return nil, NewError("库类型不支持")
}

//企业部门内容管理
func RegisterTeamLib(router *GinRouter) {

	//用户提交
	router.POSTJWT("/user/assets/submit", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &struct {
			Id    string `json:"_id"`
			Type  string //资产类型
			Scope string //资产范围 team or company
		}{}
		err := c.ShouldBindJSON(&body)
		if err != nil {
			return nil, NewError("参数不合法!")
		}
		if len(body.Scope) < 1 {
			body.Scope = "team"
		}

		if body.Scope != "team" && body.Scope != "company" {
			return nil, NewError("Scope must team or company!")
		}

		if len(body.Id) < 1 {
			return nil, NewError("Id不能为空!")
		}

		return Submit(c, apictx, body.Type, body.Id, "user", body.Scope)
	})

	router.POSTJWT("/team/assets/submit", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		body := &struct {
			Id    string `json:"_id"`
			Type  string //资产类型
			Scope string //资产范围 team or company
		}{}
		err := c.ShouldBindJSON(&body)
		if err != nil {
			return nil, NewError("参数不合法!")
		}
		if len(body.Scope) < 1 {
			body.Scope = "team"
		}

		if body.Scope != "team" && body.Scope != "company" {
			return nil, NewError("Scope must team or company!")
		}
		if len(body.Id) < 1 {
			return nil, NewError("Id不能为空!")
		}

		return Submit(c, apictx, body.Type, body.Id, "team", body.Scope)
	})

	//同步企业数据
	//router.GET("/qiye/assets/sync", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	//
	//	id := DADONG_USER_ID
	//	needPublic := true
	//
	//	qiyeId, _ := primitive.ObjectIDFromHex(id)
	//	query := map[string]interface{}{}
	//
	//	//获取所有子账号
	//	users, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
	//		CollectName: repo.CollectionUser,
	//		Page:        1,
	//		Size:        100,
	//		Query:       repo.Map{"parent": id},
	//		Project:     []string{"_id"},
	//	})
	//	if err != nil {
	//		return nil, err
	//	}
	//
	//	subAccountIds := []primitive.ObjectID{qiyeId}
	//	for _, u := range users.List {
	//		subAccountIds = append(subAccountIds, u["_id"].(primitive.ObjectID))
	//	}
	//
	//	query["userId"] = repo.Map{"$in": subAccountIds}
	//
	//	if needPublic {
	//		query["state"] = model.LibState_Publish2Qiye
	//	} else {
	//		if query["state"] == nil {
	//			query["state"] = bson.M{"$gt": model.LibState_Created}
	//		}
	//	}
	//	if query["id"] != nil {
	//		query["id"] = nil
	//	}
	//
	//	ParseCategories(query, apictx)
	//
	//	ret, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
	//		CollectName: repo.CollectionLasts,
	//		Page:        1,
	//		Size:        1000,
	//		Query:       query,
	//	})
	//
	//	//设置状态
	//
	//	//添加到数据库
	//	total := 0
	//
	//	for _, item := range ret.List {
	//		// delete(item, "_id")
	//		item["teamId"] = ""
	//		item["companyId"] = DADONG_USER_ID
	//		item["state"] = model.LibState_Publish2Qiye
	//		ret2, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), "last_company", item)
	//		fmt.Println(ret2, err)
	//		if err == nil {
	//			total += 1
	//		}
	//	}
	//
	//	return total, nil
	//})
}
