package api

import (
	"fmt"
	"strings"
)

const (
	TreeSku3dDb    = "628b4ae8633d10f8e3658557"
	Sku3dDbName    = "sku3d-tree"
	SCOPE_PERSONAl = "personal"
	SCOPE_COMPANY  = "company"
	SCOPE_PLATFORM = "platform"
)

func getAssetType(collectionName string) (string, string) {
	flags := strings.Split(collectionName, "_")
	if len(flags) != 3 {
		//两个特殊的specialmat
		if collectionName == "specialMat" {
			return "specialmat", SCOPE_PLATFORM
		}

		if collectionName == "linemat" {
			return "linemat", SCOPE_PLATFORM
		}

		if collectionName == "env3d" {
			return "env", SCOPE_PLATFORM
		}
		return "", ""
	}
	first := flags[0]
	if first == SCOPE_PLATFORM || first == SCOPE_COMPANY || first == SCOPE_PERSONAl {
		return flags[1], first
	}
	return "", ""
}

func getAssetTypeFromCollecton(collName string) (string, string) {
	asseType, scope := getAssetType(collName)
	if asseType == "mat2d" {
		asseType = "fabric2d"
	}
	if asseType == "mat" || asseType == "mats" {
		asseType = "fabric3d"
	}

	return asseType, scope
}

func getAssetTypeCollectin(isUserCompany bool, isSku3d bool, scope string, assetType string) string {
	if assetType == "specialmat" {
		return "specialMat"
	}

	if assetType == "linemat" {
		return "linemat"
	}
	if assetType == "env" {
		return "env3d"
	}

	if assetType == "fabric2d" || assetType == "imagemat" {
		assetType = "mat2d"
	}

	if assetType == "shoes" {
		assetType = "shoe"
	}

	if isSku3d {
		if assetType == "fabric3d" || assetType == "fabric" {
			assetType = "mat"
		}
		return getPlatformCollection(assetType, scope)
	}
	if isUserCompany {
		if assetType == "fabric3d" || assetType == "fabric" {
			assetType = "mats"
		}
		return getCompanyCollection(assetType, scope)
	}
	if assetType == "fabric3d" || assetType == "fabric" {
		assetType = "mat"
	}
	return getPersonCollection(assetType, scope)
}

func getPlatformCollection(assetType string, scope string) string {
	if scope == SCOPE_PLATFORM {
		scope = SCOPE_COMPANY
	}
	return fmt.Sprintf("%s_%s_%s", SCOPE_PLATFORM, assetType, scope)
}

func getCompanyCollection(assetType string, scope string) string {
	return fmt.Sprintf("%s_%s_%s", SCOPE_COMPANY, assetType, scope)
}

func getPersonCollection(assetType string, scope string) string {
	return fmt.Sprintf("%s_%s_%s", SCOPE_PERSONAl, assetType, scope)
}
